@@ -14,6 +14,7 @@ class Command:
 
     def __init__(self):
         self.help = "Run database tests with dot notation support"
+        self._pg_container = None
 
     async def safe_shutdown(self):
         """Safely shutdown database connections with timeout."""
@@ -32,24 +33,91 @@ class Command:
         """Run shutdown in the current event loop context."""
         await self.safe_shutdown()
 
-    def _bootstrap_sqlite(self):
-        # Use in-memory sqlite as the default test DB
-        import os
-        from neutronapi.db import setup_databases
-        os.environ['TESTING'] = '1'  # Ensure ConnectionsManager prefers :memory:
+    async def _has_existing_postgres_server(self, db_config: dict) -> bool:
+        """Check if PostgreSQL server is already running and accessible."""
         try:
-            from apps.settings import DATABASES  # Optional, if running inside a project
-            setup_databases(DATABASES)
-            return
-        except Exception:
-            # Fall back to a guaranteed in-memory configuration even if a manager already exists
-            test_config = {
-                'default': {
-                    'ENGINE': 'aiosqlite',
-                    'NAME': ':memory:',
-                }
-            }
-            setup_databases(test_config)
+            import asyncpg
+            conn = await asyncpg.connect(
+                host=db_config.get('HOST', 'localhost'),
+                port=db_config.get('PORT', 5432),
+                database='postgres',  # Connect to default postgres DB to test connection
+                user=db_config.get('USER', 'postgres'),
+                password=db_config.get('PASSWORD', ''),
+            )
+            await conn.close()
+            return True
+        except:
+            return False
+
+    async def _setup_test_database(self, db_config: dict):
+        """Create a test database on existing PostgreSQL server."""
+        import asyncpg
+        test_db_name = f"test_{db_config.get('NAME', 'neutronapi')}"
+        
+        # Update settings to use test database
+        from neutronapi.conf import settings
+        if hasattr(settings, '_settings') and 'DATABASES' in settings._settings:
+            settings._settings['DATABASES']['default']['NAME'] = test_db_name
+            
+        try:
+            # Connect to postgres db to manage test databases
+            conn = await asyncpg.connect(
+                host=db_config.get('HOST', 'localhost'),
+                port=db_config.get('PORT', 5432),
+                database='postgres',
+                user=db_config.get('USER', 'postgres'),
+                password=db_config.get('PASSWORD', ''),
+            )
+            
+            # Clean up any dangling test databases from previous runs
+            print("Cleaning up any dangling test databases...")
+            dangling_dbs = await conn.fetch(
+                "SELECT datname FROM pg_database WHERE datname LIKE 'test_%'"
+            )
+            for db_row in dangling_dbs:
+                db_name = db_row['datname']
+                print(f"Dropping dangling test database: {db_name}")
+                await conn.execute(f'DROP DATABASE IF EXISTS "{db_name}"')
+            
+            # Create our test database
+            await conn.execute(f'CREATE DATABASE "{test_db_name}"')
+            await conn.close()
+            
+            print(f"Created clean test database: {test_db_name}")
+        except Exception as e:
+            print(f"Warning: Could not create test database: {e}")
+
+    async def _setup_test_sqlite(self, db_config: dict):
+        """Setup in-memory SQLite for tests."""
+        from neutronapi.conf import settings
+        if hasattr(settings, '_settings') and 'DATABASES' in settings._settings:
+            settings._settings['DATABASES']['default']['NAME'] = ':memory:'
+            print("Using in-memory SQLite for tests")
+
+    async def _cleanup_test_database(self):
+        """Clean up test database if we created one."""
+        try:
+            from neutronapi.conf import settings
+            if hasattr(settings, '_settings') and 'DATABASES' in settings._settings:
+                db_config = settings._settings['DATABASES']['default']
+                db_name = db_config.get('NAME', '')
+                engine = db_config.get('ENGINE', '').lower()
+                
+                if engine == 'asyncpg' and db_name.startswith('test_') and not self._pg_container:
+                    # Only cleanup if we created a test database on existing server
+                    import asyncpg
+                    conn = await asyncpg.connect(
+                        host=db_config.get('HOST', 'localhost'),
+                        port=db_config.get('PORT', 5432),
+                        database='postgres',
+                        user=db_config.get('USER', 'postgres'),
+                        password=db_config.get('PASSWORD', ''),
+                    )
+                    await conn.execute(f'DROP DATABASE IF EXISTS "{db_name}"')
+                    await conn.close()
+                    print(f"Cleaned up test database: {db_name}")
+        except Exception as e:
+            print(f"Warning: Could not cleanup test database: {e}")
 
     async def _run_async(self, *cmd: str, timeout: Optional[float] = None) -> Tuple[int, str, str]:
         """Run a subprocess asynchronously and capture output."""
@@ -158,28 +226,22 @@ class Command:
             print(f"Error waiting for PostgreSQL: {e}")
             return False
 
-        # Configure environment for tests
+        # Update settings with container connection info
         try:
-            os.environ['TESTING'] = '1'
-            os.environ['PGHOST'] = host
-            os.environ['PGPORT'] = str(port)
-            os.environ['PGDATABASE'] = dbname
-            os.environ['PGUSER'] = user
-            os.environ['PGPASSWORD'] = password
-
-            # Force database system to use the new config
-            from neutronapi.db import setup_databases
-            test_config = {
-                'default': {
-                    'ENGINE': 'asyncpg',
+            from neutronapi.conf import settings
+            if hasattr(settings, '_settings') and 'DATABASES' in settings._settings:
+                # Update the existing database config with container details
+                db_config = settings._settings['DATABASES']['default']
+                db_config.update({
                     'HOST': host,
                     'PORT': port,
                     'NAME': dbname,
                     'USER': user,
                     'PASSWORD': password,
-                }
-            }
-            setup_databases(test_config)
+                })
+                print(f'Updated database config: {db_config}')
+            else:
+                print("Warning: Could not update database settings")
 
             print(f'✓ PostgreSQL ready at {host}:{port}/{dbname}')
             return True
@@ -221,22 +283,28 @@ class Command:
             print(self.handle.__doc__)
             return
 
-        # No project-specific env flags; keep behavior generic
-
-        # Bootstrap test databases
-        provider_env = os.getenv('DATABASE_PROVIDER', '').lower().strip()
-        if provider_env in ('asyncpg', 'postgres', 'postgresql'):
-            print('Bootstrapping PostgreSQL test database...')
-            try:
-                success = await self._bootstrap_postgres()
-            except Exception:
-                success = False
-            if not success:
-                print('Failed to bootstrap PostgreSQL, falling back to SQLite')
-                self._bootstrap_sqlite()
+        # Check database configuration and setup test database
+        from neutronapi.conf import settings
+        if hasattr(settings, 'DATABASES'):
+            db_config = settings.DATABASES.get('default', {})
+            engine = db_config.get('ENGINE', '').lower()
+            
+            if engine == 'asyncpg':
+                # For PostgreSQL, check if user has existing server or use Docker
+                if not await self._has_existing_postgres_server(db_config):
+                    print("No existing PostgreSQL server found, bootstrapping test database container...")
+                    success = await self._bootstrap_postgres()
+                    if not success:
+                        print("Failed to bootstrap PostgreSQL container. Tests may fail.")
+                        return 1
+                else:
+                    print("Using existing PostgreSQL server for tests...")
+                    await self._setup_test_database(db_config)
+            elif engine == 'aiosqlite':
+                # For SQLite, always use in-memory test database
+                await self._setup_test_sqlite(db_config)
         else:
-            print('Bootstrapping SQLite in-memory test database...')
-            self._bootstrap_sqlite()
+            print("Warning: No DATABASES configuration found")
 
         # Apply project migrations (if any) using the file-based tracker
         async def apply_project_migrations():
@@ -272,13 +340,18 @@ class Command:
         except Exception:
             pass
 
-        # Bootstrap internal test models (used by this package's own tests)
+        # Bootstrap internal test models (only when developing neutronapi itself)
         async def bootstrap_test_models():
             try:
+                # Only bootstrap test models if we're in the neutronapi development environment
+                # This is indicated by the presence of neutronapi source code in the current directory
+                if not os.path.isdir("neutronapi") or not os.path.isfile("neutronapi/__init__.py"):
+                    return  # We're not in the neutronapi development environment
+                
                 from neutronapi.db.migrations import CreateModel
                 from neutronapi.db.connection import get_databases
                 
-                # Try to discover test models from neutronapi.tests.db
+                # Try to discover test models from local neutronapi.tests.db
                 try:
                     from neutronapi.tests.db.test_models import TestUser
                     from neutronapi.tests.db.test_queryset import TestObject
@@ -289,7 +362,7 @@ class Command:
                     connection = await get_databases().get_connection('default')
                     
                     for model_cls in test_models:
-                        create_operation = CreateModel(f'neutronapi.{model_cls.__name__}', model_cls._fields)
+                        create_operation = CreateModel(f'neutronapi.{model_cls.__name__}', model_cls._neutronapi_fields_)
                         await create_operation.database_forwards(
                             app_label='neutronapi',
                             provider=connection.provider,
@@ -300,8 +373,9 @@ class Command:
                     
                     print(f"✓ Bootstrapped {len(test_models)} internal test models")
                     
-                except ImportError as e:
-                    print(f"Note: Test models not found, skipping bootstrap: {e}")
+                except ImportError:
+                    # Silently skip if test models not available
+                    pass
                 except Exception as e:
                     print(f"Warning: Failed to bootstrap test models: {e}")
                     
@@ -334,7 +408,10 @@ class Command:
                 else:
                     filtered_args.append(a)
 
-            runner = unittest.TextTestRunner(verbosity=verbosity, stream=sys.stderr)
+            # Force unbuffered output for real-time test results
+            sys.stdout.reconfigure(line_buffering=True)
+            sys.stderr.reconfigure(line_buffering=True)
+            runner = unittest.TextTestRunner(verbosity=verbosity, stream=sys.stderr, buffer=False)
             suite = unittest.TestSuite()
 
             # Start coverage if requested or env flag set
@@ -356,12 +433,20 @@ class Command:
             def add_target(target: str):
                 # If target is an app label (directory in apps/ or 'core')
                 if os.path.isdir(os.path.join("apps", target, "tests")):
-                    discovered = loader.discover(os.path.join("apps", target, "tests"), pattern="test_*.py")
+                    # Add the apps directory to sys.path so nested test modules can be imported
+                    apps_dir = "apps"
+                    if apps_dir not in sys.path:
+                        sys.path.insert(0, apps_dir)
+                    # Start discovery from apps directory with proper top_level_dir
+                    discovered = loader.discover(
+                        start_dir=os.path.join("apps", target, "tests"),
+                        pattern="test_*.py",
+                        top_level_dir="apps"
+                    )
                     suite.addTests(discovered)
                     return
-                if target == "core" and (os.path.isdir("core/tests") or os.path.isdir("neutronapi/tests")):
-                    test_root = "core/tests" if os.path.isdir("core/tests") else "neutronapi/tests"
-                    discovered = loader.discover(test_root, pattern="test_*.py")
+                if target == "core" and os.path.isdir("core/tests"):
+                    discovered = loader.discover("core/tests", pattern="test_*.py")
                     suite.addTests(discovered)
                     return
 
@@ -378,16 +463,20 @@ class Command:
                 for target in filtered_args:
                     add_target(target)
             else:
-                # Default: discover core and all apps/*/tests
+                # Default: discover all apps/*/tests (project-specific only)
                 test_dirs = []
 
-                # Support both legacy core/tests and current neutronapi/tests
+                # Only look for project tests, not installed package tests
+                # Support legacy core/tests if it exists in the current project
                 if os.path.isdir("core/tests"):
                     test_dirs.append("core/tests")
-                if os.path.isdir("neutronapi/tests"):
-                    test_dirs.append("neutronapi/tests")
 
                 if os.path.isdir("apps"):
+                    # Add apps to sys.path for proper module resolution
+                    apps_dir = "apps"
+                    if apps_dir not in sys.path:
+                        sys.path.insert(0, apps_dir)
+                    
                     for app_name in os.listdir("apps"):
                         app_tests_dir = os.path.join("apps", app_name, "tests")
                         if os.path.isdir(app_tests_dir):
@@ -395,7 +484,11 @@ class Command:
 
                 if test_dirs:
                     for test_dir in test_dirs:
-                        discovered = loader.discover(test_dir, pattern="test_*.py")
+                        # Use proper top_level_dir for apps
+                        if test_dir.startswith("apps"):
+                            discovered = loader.discover(test_dir, pattern="test_*.py", top_level_dir="apps")
+                        else:
+                            discovered = loader.discover(test_dir, pattern="test_*.py")
                         suite.addTests(discovered)
                 else:
                     suite = loader.discover(".", pattern="test_*.py")
@@ -432,12 +525,31 @@ class Command:
                         cov.html_report(directory='htmlcov')
             except Exception as e:
                 print(f"Warning: coverage reporting failed: {e}")
-            await self.run_forced_shutdown()
+            
+            # Cleanup with timeout
+            try:
+                await asyncio.wait_for(self.run_forced_shutdown(), timeout=3.0)
+            except asyncio.TimeoutError:
+                print("Warning: Database cleanup timed out")
+            except Exception as e:
+                print(f"Warning: Database cleanup failed: {e}")
+            
+            # Cleanup test database if needed
+            try:
+                await asyncio.wait_for(self._cleanup_test_database(), timeout=3.0)
+            except asyncio.TimeoutError:
+                print("Warning: Test database cleanup timed out")
+            except Exception as e:
+                print(f"Warning: Test database cleanup failed: {e}")
+            
             # Best-effort: stop ephemeral postgres if we started it
             try:
-                await self._teardown_postgres()
+                await asyncio.wait_for(self._teardown_postgres(), timeout=3.0)
+            except asyncio.TimeoutError:
+                print("Warning: PostgreSQL cleanup timed out")
             except Exception:
                 pass
+                
             # Default: hard-exit to ensure full shutdown (no lingering loops/threads)
             # For programmatic callers/tests, set NEUTRONAPI_TEST_RETURN=1 to receive the code instead.
             if os.getenv('NEUTRONAPI_TEST_RETURN', '0') == '1':
